#load necessary R packages
library(Amelia); library(psych); library(mitools); library(multilevel); library(Hmisc); library(lmtest); library(car)
#set your working directory to whichever folder you downloaded the analysis files into - make sure the data files are in the same folder
setwd('PATH NAME TO FOLDER HERE')

source('Miles and Vaisey 2015 - functions.R', chdir = TRUE)
#read in Measuring Morality data file
load("MMdata_merged.rdata")
#rename the data file to "d"
d=MMdata_merged
#code missing
d[d<0]=NA

### code morality scales for each theory
#Hunter
d$moralrelig=8-rowMeans(cbind(d$MR2, d$MR3), na.rm=T)  #reverse code  so that higher is more belief in divine authority
d$absolutism=with(d, as.numeric(scale(rowMeans(cbind(EPQ1, EPQ2, EPQ3, MR1), na.rm=T))))

#Lakoff
d$revL12 = 8-d$L12
d$lak_npfam = with(d, rowMeans(cbind(L9, L10, L11, revL12), na.rm=T))
d$lak_sffam = with(d, rowMeans(cbind(L13, L14, L15, L16), na.rm=T))

#Scwartz
#reverse code items
pvq.mat=7-d[,grep("SV", names(d))]
colnames(pvq.mat)=paste("rev",colnames(pvq.mat),  sep="")
d=cbind(d, pvq.mat)

d$sv_conform = with(d, rowMeans(cbind(revSV7, revSV16), na.rm=T))
d$sv_trad = with(d, rowMeans(cbind(revSV9, revSV20), na.rm=T))
d$sv_benev = with(d, rowMeans(cbind(revSV12, revSV18), na.rm=T))
d$sv_univ = with(d, rowMeans(cbind(revSV3, revSV8, revSV19), na.rm=T))
d$sv_selfd = with(d, rowMeans(cbind(revSV1, revSV11), na.rm=T))
d$sv_stim = with(d, rowMeans(cbind(revSV6, revSV15), na.rm=T))
d$sv_hedon = with(d, rowMeans(cbind(revSV10, revSV21), na.rm=T))
d$sv_achieve = with(d, rowMeans(cbind(revSV4, revSV13), na.rm=T))
d$sv_power = with(d, rowMeans(cbind(revSV2, revSV17), na.rm=T))
d$sv_security = with(d, rowMeans(cbind(revSV5, revSV14), na.rm=T))
#do not adjust for scale response tendencies as is typical, as this creates exact dependencies in factor analysis 

#politics variables
d$libcons = d$pppa0012
d$party = d$partyid7
d$party.rev = 8-d$party

#demographic variables - both for analyses and to help with imputation
d$female = ifelse(d$ppgender==2, 1, 0)
d$educ.cat = d$ppeducat
# split off post-BA education
d$educ.cat = ifelse(d$ppeduc>12, 5, d$educ.cat)
#other/biracial category
d$black = ifelse(d$ppethm==2, 1, 0)
d$otherrace = ifelse(d$ppethm==3 | d$ppethm==5, 1, 0)
d$hispanic = ifelse(d$ppethm==4, 1, 0)
#age
d$age=d$ppage
d$age.sq = d$age^2
d$married = ifelse(d$ppmarit==1, 1, 0)
d$south = ifelse(d$PPREG4==3, 1, 0)
#religion
d$consrel = ifelse(d$pppa0070==1 | d$pppa0070==4 | d$pppa0070==9, 1, 0)
d$other.christian = ifelse(d$pppa0070==11, 1, 0)
d$norelig = ifelse(d$pppa0070==13, 1, 0)
d$working = ifelse(d$ppwork<3, 1, 0)
d$retired = ifelse(d$ppwork==5, 1, 0)
d$attend = 6-d$ppp20072  #recode so higher is more attendance
d$hhsize = ifelse(d$pphhsize > 8, 8, d$pphhsize) #top code so that those with more than 8 are coded as 8
d$poli.interest = 5-d$pppa0035
d$religimport = as.numeric(scale(8-rowMeans(cbind(d$RFQ1, d$RFQ2), na.rm=T)))
d$internet = d$ppnet
#other variables that might improve imputation
d$compassion = with(d, as.numeric(scale(rowMeans(cbind(DPES13, DPES14, DPES15)))))
d$numberone = as.numeric(scale(d$IS2))

#remove cases not being used - missing on any variables that cannot be imputed, see last paragraph in Appendix A
missobs=is.na(d$lak_npfam) | is.na(d$lak_sffam) | is.na(d$moralrelig)
d=d[!missobs,]

#alphas for the measures - in original data sets since we don't have individual items after imputation
#Hunter
cronbach(cbind(d$MR2, d$MR3)) 
with(d, cronbach(cbind(EPQ1, EPQ2, EPQ3, MR1)))

#Lakoff
with(d, cronbach(cbind(L9, L10, L11, revL12))) #nurturant parent
with(d, cronbach(cbind(L13, L14, L15, L16))) #strict father

#values
with(d, cronbach(cbind(revSV7, revSV16))) #conformity
with(d, cronbach(cbind(revSV9, revSV20))) #tradition
with(d, cronbach(cbind(revSV12, revSV18))) #benevolence
with(d, cronbach(cbind(revSV3, revSV8, revSV19))) #universalism
with(d, cronbach(cbind(revSV1, revSV11))) #self-direction
with(d, cronbach(cbind(revSV6, revSV15))) #stimulation
with(d, cronbach(cbind(revSV10, revSV21))) #hedonism
with(d, cronbach(cbind(revSV4, revSV13))) #achievement
with(d, cronbach(cbind(revSV2, revSV17))) #power
with(d, cronbach(cbind(revSV5, revSV14))) #security

#read in secondary data set with MFQ20 measures
load("ggdata.rdata")
d2=ggdata
rm(ggdata)

#Morality variables
#Hunter
d2$absolutism = as.numeric(scale(d2$q43)) #just 1 item to more for data d

#code MFQ
#recode all variables to a 0 to 5 scale, consistent with the original coding
d2$q27 = recode(d2$q27, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q28 = recode(d2$q28, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q29 = recode(d2$q29, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q30 = recode(d2$q30, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q31 = recode(d2$q31, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q32 = recode(d2$q32, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q33 = recode(d2$q33, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q34 = recode(d2$q34, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q35 = recode(d2$q35, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q36 = recode(d2$q36, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52a = recode(d2$q52a, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52b = recode(d2$q52b, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52c = recode(d2$q52c, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52d = recode(d2$q52d, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52e = recode(d2$q52e, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52f = recode(d2$q52f, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52g = recode(d2$q52g, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52h = recode(d2$q52h, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52i = recode(d2$q52i, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52j = recode(d2$q52j, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")
d2$q52k = recode(d2$q52k, "1=0; 2=1; 3=2; 4=3; 5=4; 6=5; else=NA")

#remove those who answered the catch question in one of the top three categories
d2$catch.wrong=with(d2, ifelse(q52a > 2, 1, 0))

d2$q27 = ifelse(d2$catch.wrong==1, NA, d2$q27)
d2$q28 = ifelse(d2$catch.wrong==1, NA, d2$q28)
d2$q29 = ifelse(d2$catch.wrong==1, NA, d2$q29)
d2$q30 = ifelse(d2$catch.wrong==1, NA, d2$q30)
d2$q31 = ifelse(d2$catch.wrong==1, NA, d2$q31)
d2$q32 = ifelse(d2$catch.wrong==1, NA, d2$q32)
d2$q33 = ifelse(d2$catch.wrong==1, NA, d2$q33)
d2$q34 = ifelse(d2$catch.wrong==1, NA, d2$q34)
d2$q35 = ifelse(d2$catch.wrong==1, NA, d2$q35)
d2$q36 = ifelse(d2$catch.wrong==1, NA, d2$q36)
d2$q52a = ifelse(d2$catch.wrong==1, NA, d2$q52a)
d2$q52b = ifelse(d2$catch.wrong==1, NA, d2$q52b)
d2$q52c = ifelse(d2$catch.wrong==1, NA, d2$q52c)
d2$q52d = ifelse(d2$catch.wrong==1, NA, d2$q52d)
d2$q52e = ifelse(d2$catch.wrong==1, NA, d2$q52e)
d2$q52f = ifelse(d2$catch.wrong==1, NA, d2$q52f)
d2$q52g = ifelse(d2$catch.wrong==1, NA, d2$q52g)
d2$q52h = ifelse(d2$catch.wrong==1, NA, d2$q52h)
d2$q52i = ifelse(d2$catch.wrong==1, NA, d2$q52i)
d2$q52j = ifelse(d2$catch.wrong==1, NA, d2$q52j)
d2$q52k = ifelse(d2$catch.wrong==1, NA, d2$q52k)

#Schwartz values
d2$sv.univ = 7-d2$q51a
d2$sv.benev = 7-d2$q51h
d2$sv.conform = 7-d2$q51f
d2$sv.trad = 7-d2$q51b 
d2$sv.selfd = 7-d2$q51j 
d2$sv.stim = 7-d2$q51c 
d2$sv.power = 7-d2$q51i 
d2$sv.achieve = 7-d2$q51g 
d2$sv.hedon = 7-d2$q51e 
d2$sv.security = 7-d2$q51d 
#don't adjust for scale response tendencies or factor analysis won't work

#political variables
d2$libcons = d2$pppa0015
d2$party.rev=8-d2$partyid7

#demographic variables
d2$female = ifelse(d2$ppgender==2, 1, 0)
d2$black = ifelse(d2$ppethm==2, 1, 0)
d2$otherrace = ifelse(d2$ppethm==3 | d2$ppethm==5, 1, 0)
d2$hispanic = ifelse(d2$ppethm==4, 1, 0)
d2$educ.cat=factor(d2$ppeducat)
# split off post-BA education
d2$educ.cat = ifelse(d2$ppeduc>12, 5, d2$educ.cat)
d2$age = d2$ppage
d2$age.sq = d2$age^2
d2$married = ifelse(d2$ppmarit==1, 1, 0)
d2$south = ifelse(d2$ppreg4==3, 1, 0)
d2$consrel = ifelse(d2$xrelig==1 | d2$xrelig==4 | d2$xrelig==9, 1, 0)
d2$other.christian = ifelse(d2$xrelig==11, 1, 0)
d2$norelig = ifelse(d2$xrelig==13, 1, 0)
d2$working = ifelse(d2$ppwork<3, 1, 0)
d2$retired = ifelse(d2$ppwork==5, 1, 0)
d2$attend = ifelse(d2$xchurch==7, NA, d2$xchurch)
#recode so higher is more attendance
d2$attend = 6-d2$attend
d2$hhsize = ifelse(d2$pphhsize > 8, 8, d2$pphhsize) #top code so that those with more than 8 are coded as 8
d2$poli.interest = 5-d2$pppa0004
d2$religimport = as.numeric(scale(6-d2$q48)) 
d2$internet = d2$ppnet
#other variables that might improve imputation
#scale of most highly correlated compassion items
d2$compassion = with(d2, as.numeric(scale(rowMeans(cbind(q1, q2, q5, q6, q9, q10)))))
d2$numberone = with(d2, as.numeric(scale(q11)))

#prep two data sets for combining
#variables to add in after imputation (model won't impute if you have them in there, because they create a situation where you have no data, since they are all missing in one data set, and the MFT variables are all missing in the other)
addvars=c("moralrelig", "lak_npfam", "lak_sffam")

d.prep=d[,c("female", "educ.cat", "age", "age.sq", "married", "south", "consrel", "libcons", "party.rev", "ppincimp", "norelig", "other.christian", "black", "otherrace", "hispanic", "hhsize", "working", "retired", "attend", "poli.interest", "absolutism", "religimport", "weight", "internet", "compassion", "numberone", grep("sv_", names(d), value=T))]
#add extra columns for the missing MFT measures
d.prep=cbind(d.prep, q27=NA, q28=NA, q52b=NA, q52c=NA, q29=NA, q30=NA, q52d=NA, q52e=NA, q31=NA, q32=NA, q52f=NA, q52g=NA, q33=NA, q34=NA, q52h=NA, q52i=NA, q35=NA, q36=NA, q52j=NA, q52k=NA)
d.prep=d.prep[,order(names(d.prep))]

d2.prep=d2[,c("female", "educ.cat", "age", "age.sq", "married", "south", "consrel", grep("sv.", names(d2), value=T, fixed=T), "libcons", "party.rev", "ppincimp", "weight", "norelig", "other.christian", "absolutism", "religimport", "working", "retired", "black", "otherrace", "hispanic", "hhsize", "poli.interest", "compassion", "internet", "numberone", "attend", "q27", "q28", "q29", "q30", "q31", "q32", "q33", "q34", "q35", "q36", "q52b", "q52c", "q52d", "q52e", "q52f", "q52g", "q52h", "q52i", "q52j", "q52k")]
d2.prep=d2.prep[,order(names(d2.prep))]

names(d.prep)=names(d2.prep)
d3=rbind(d.prep, d2.prep)
d3$educ.cat=as.factor(d3$educ.cat)
#indicator for whether observation came from Measuring Morality data
d3$mmsurvey = c(rep(1, dim(d.prep)[1]), rep(0, dim(d2.prep)[1]))
#education variables for testing
d3$hs = ifelse(d3$educ.cat==2, 1, 0)
d3$somecoll = ifelse(d3$educ.cat==3, 1, 0)
d3$ba = ifelse(d3$educ.cat==4, 1, 0)
d3$maplus = ifelse(d3$educ.cat==5, 1, 0)

# check to make sure that the variables that are assumed to tap  the same concepts plausibly are doing that based on how they relate to other variables 
varlist=with(d3, list(female, black, hispanic, otherrace, hs, somecoll, ba, maplus, ppincimp, age, married, south, consrel, other.christian, norelig, libcons, party.rev, hhsize, working, retired, internet, poli.interest, religimport, attend, absolutism, compassion, numberone, sv.conform, sv.trad, sv.benev, sv.univ, sv.hedon, sv.selfd, sv.stim, sv.power, sv.achieve, sv.security))
names(varlist)=c("female", "black", "hispanic", "otherrace", "hs", "somecoll", "ba", "maplus", "ppincimp", "age", "married", "south", "consrel", "other.christian", "norelig", "libcons", "party.rev", "hhsize", "working", "retired", "internet", "poli.interest", "religimport", "attend", "absolutism", "compassion", "numberone", "sv.conform", "sv.trad", "sv.benev", "sv.univ", "sv.hedon", "sv.selfd", "sv.stim", "sv.power", "sv.achieve", "sv.security")

#extract pairs of variables whose linear bivariate relationship differs across surveys
which.notsame=compare.surveys(varlist, bicdiff=2)[-1,]
#percent of bivariate relationships that aren't the same between data sets
perc.bivars=table(c(which.notsame[,1], which.notsame[,2]))/(length(varlist)-1)
perc.bivars
perc.all = rep(0, length(varlist))
names(perc.all)=names(varlist)
perc.all[match(names(perc.bivars), names(perc.all))]=perc.bivars

#plot the results
par(mar=c(6,4,1,2))
bp=barplot(perc.all, xaxt="n", ylab="Percent", las=1, ylim=c(0,max(perc.all)+max(perc.all)/8), las=2, cex.axis=.6, cex.lab=.6)
abline(h=seq(0,1,.1), col="grey")
abline(h=.33, lty=2)
barplot(perc.all, xaxt="n", add=T, yaxt="n")
varlabs=c("Female", "Black", "Hispanic", "Other race", "High school", "Some college", "BA", "MA or above","Income", "Age", "Married", "South", "Conservative Christian", "Other Christian", "No religion", "Conservative", "Party", "HH size", "Working", "Retired", "Internet", "Interest in politics", "Importance of religion", "Religious service attendance", "Absolutism", "Compassion", "Putting self first", "Conformity", "Tradition", "Benevolence", "Universalism", "Hedonism", "Self-direction", "Stimulation", "Power", "Achievement", "Security" )
text(bp[,1]+1, y=-.02, labels=varlabs, srt=45, pos=2, xpd=T, offset=1, cex=.6)


#based on this, want to remove a few variables with higher rates of non-agreement, as well as the education variables which were imputed using educ.cat
d3=d3[,-which(colnames(d3) %in% c("attend", "internet", "hs", "somecoll", "ba", "maplus"))]

#need to transform variables prior to imputation?
hist(d3$age)  #fine
hist(d3$absolutism)  #fine
hist(d3$religimport) #fine
hist(d3$party.rev) #fine
hist(d3$libcons) #fine
hist(d3$sv.conform) #fine
hist(d3$weight) #try a log transformation
hist(log(d3$weight)) #use log transformation
hist(d3$ppincimp) #fine
hist(d3$hhsize) # try a log transformation
hist(log(d3$hhsize)) #use log transformation
hist(sqrt(d3$hhsize))
hist(d3$poli.interest) #fine
hist(d3$compassion)  #fine
hist(d3$numberone)  #fine
hist(d3$q32) #fine

logvars=c("weight", "hhsize")

#number of imputations should at least equal the highest rate of missingness
apply(d3, 2, function(x) sum(is.na(x)))/dim(d3)[1]
#so at least 65 imputations for the 65% missing data rate for MFQ variables

#impute, m=200 
dimp=amelia(d3, m=200, ords=c("educ.cat"), idvars=c("mmsurvey"), logs=logvars, p2s=1)

#trim back down to MM survey
dimp$imputations=lapply(dimp$imputations, function(x) subset(x, x$mmsurvey==1))

#put in an imputation list so we can use tools from mitools package with it
imps=imputationList(dimp$imputations)

#add in other variables 
imps$imputations = lapply(imps$imputations, function(x){cbind(x, d[,addvars]) })
#code outcome
imps$imputations=lapply(imps$imputations, function(x){x$conserv=rowMeans(cbind(x$libcons, x$party.rev)); return(x)})
#create an indicator for those originally missing on outcome
conserv.miss=ifelse(is.na(d$libcons) | is.na(d$party.rev), 1, 0)
#recode a few variables
imps$imputations=lapply(imps$imputations, function(x){x$hs=ifelse(x$educ.cat==2, 1, 0); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$somecoll=ifelse(x$educ.cat==3, 1, 0); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$ba=ifelse(x$educ.cat==4, 1, 0); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$maplus=ifelse(x$educ.cat==5, 1, 0); return(x)})
#code MFQ20 subscales
imps$imputations=lapply(imps$imputations, function(x){x$mfq.harm=rowMeans(cbind(x$q27, x$q28, x$q52b, x$q52c)); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$mfq.fair=rowMeans(cbind(x$q29, x$q30, x$q52d, x$q52e)); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$mfq.ingroup=rowMeans(cbind(x$q31, x$q32, x$q52f, x$q52g)); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$mfq.auth=rowMeans(cbind(x$q33, x$q34, x$q52h, x$q52i)); return(x)})
imps$imputations=lapply(imps$imputations, function(x){x$mfq.pure=rowMeans(cbind(x$q35, x$q36, x$q52j, x$q52k)); return(x)})


#get alphas for MFQ20
#care
mean(sapply(imps$imputations, function(x) cronbach(cbind(x$q27, x$q28, x$q52b, x$q52c))$Alpha)) 
#fair
mean(sapply(imps$imputations, function(x) cronbach(cbind(x$q29, x$q30, x$q52d, x$q52e))$Alpha)) 
#ingroup
mean(sapply(imps$imputations, function(x) cronbach(cbind(x$q31, x$q32, x$q52f, x$q52g))$Alpha)) 
#authority
mean(sapply(imps$imputations, function(x) cronbach(cbind(x$q33, x$q34, x$q52h, x$q52i))$Alpha)) 
#purity
mean(sapply(imps$imputations, function(x) cronbach(cbind(x$q35, x$q36, x$q52j, x$q52k))$Alpha)) 

#descriptives (weighted) - Table 2
whichvars=c("conserv", "libcons", "party.rev", "mfq.harm", "mfq.fair", "mfq.ingroup", "mfq.auth", "mfq.pure", "moralrelig", "absolutism", grep("sv.", names(d2), value=T, fixed=T), "lak_npfam", "lak_sffam", grep("mfq.", names(d2), value=T, fixed=T), "female","black", "hispanic", "otherrace", "hs", "somecoll", "ba", "maplus", "ppincimp", "age", "married", "south", "consrel", "other.christian", "norelig" )

#calculate in each data set then combine
desc.info=lapply(imps$imputations, get.desc, vars=whichvars)
desc.mat=mi.pointest(desc.info)

#set a few dummy variable SDs to 0 (missed by code above because they contain imputed values)
desc.mat[c("consrel", "other.christian", "norelig"), "sds"]=NA
desc.mat

#correlations of morality variables - Table 1
whichvars=whichvars=c("moralrelig", "absolutism", "lak_npfam", "lak_sffam", grep("mfq.", names(d2), value=T, fixed=T), grep("sv.", names(d2), value=T, fixed=T))
cormats=lapply(imps$imputations, function(x) cor(x[,whichvars], use="complete"))
cormat=mi.pointest(cormats)
#get a lower triangular matrix
cormat[upper.tri(cormat)]=NA
cormat

## can we extract a few underlying factors that cross-cut theoretical traditions? - Table 3
sort.names=c("moralrelig", "absolutism", "lak_npfam", "lak_sffam", "mfq.harm", "mfq.fair", "mfq.ingroup", "mfq.auth", "mfq.pure",  "sv.conform", "sv.trad", "sv.benev", "sv.univ", "sv.selfd", "sv.stim", "sv.hedon", "sv.achieve", "sv.power", "sv.security")
moral.dat=lapply(imps$imputations, function(x) x[,sort.names])
#check needed number of factors by VSS and parallel - see Hayton et al 2004, Revelle and Rocklin 1979
#note the complexity in VSS is the number of non-zero loadings we expect - i.e., do we theoretically expect the item  to load onto one or more than one factor?
# using just one imputed data set to sort through results below, but note that using just one imputed data set gives unbiased estimates - the multiple data sets are to help with uncertainty (for SE's)
vss(moral.dat[[1]], rotate="promax")  #suggests in the 2-4 range for complexity 2
vss(moral.dat[[1]], rotate="oblimin") #suggests in the 2-4 range for complexity 2
fa.parallel(moral.dat[[1]], fa="fa", n.iter=50)  #suggests 5 but major action in the first 3-4 
#so which solution is most theoretically interpretable?
fa(moral.dat[[1]], nfactors=2, rotate="promax", scores="tenBerge") #conflates care and fairness with order-related items
fa(moral.dat[[1]], nfactors=3, rotate="promax", scores="tenBerge") #authority and purity with order items, an other-focus factor, a self-focus factor
fa(moral.dat[[1]], nfactors=4, rotate="promax", scores="tenBerge") #creates a MFQ method factor that pulls items away from theoretical homes (e.g., authority and purity not with order, harm and fairness not with other-focus)
fa(moral.dat[[1]], nfactors=5, rotate="promax", scores="tenBerge") #retains methods factor, a hard to interpret 5th factor

#3 factors seems most interpretable - can other rotations make it cleaner?
m=fa(moral.dat[[1]], nfactors=3, rotate="promax", scores="tenBerge") #original
m1=fa(moral.dat[[1]], nfactors=3, rotate="oblimin", scores="tenBerge")  #yes
m2=fa(moral.dat[[1]], nfactors=3, rotate="simplimax", scores="tenBerge")  #no, factors much less interpretable
m3=fa(moral.dat[[1]], nfactors=3, rotate="bentlerQ", scores="tenBerge")  #yes
m4=fa(moral.dat[[1]], nfactors=3, rotate="geominQ", scores="tenBerge")  #yes, but less clean
m5=fa(moral.dat[[1]], nfactors=3, rotate="biquartimin", scores="tenBerge")  #sort of, but much less interpretable
m6=fa(moral.dat[[1]], nfactors=3, rotate="cluster", scores="tenBerge")  #yes, might be the cleanest

#compare
print(loadings(m), cutoff=.3)
print(loadings(m1), cutoff=.3) #m is slightly cleaner (stronger loadings)
print(loadings(m3), cutoff=.3) #m is slightly cleaner (stronger loadings)
print(loadings(m6), cutoff=.3) #m is slightly cleaner (stronger loadings)
#all give about the same answer, so will go with promax because it is familiar

famoral1=fa(moral.dat[[1]], nfactors=3, rotate="promax", scores="Bartlett")
famoral2=fa(moral.dat[[1]], nfactors=3, rotate="promax", scores="tenBerge")
famoral3=fa(moral.dat[[1]], nfactors=3, rotate="promax", scores="regression")
scoresmat=cbind(famoral1$scores, famoral2$scores, famoral3$scores)
cormat=cor(scoresmat, use="complete.obs")
rownames(cormat)=colnames(cormat)=c("1.Bart", "2.Bart", "3.Bart", "1.tB", "2.tB", "3.tB","1.reg", "2.reg", "3.reg")
round(cormat, 2)
#so ten Berge are highly correlated with both Bartlett and regression scores
#ten Berge preserves correlations among factors in the scores - also, they are highly correlated with the latent factors, so we'll use those (see Grice 2001)

fa.anals=lapply(imps$imputations, mi.fa, nfacts=3)
fa.anals=lapply(fa.anals, rename.factors) #rename factors so they are consistent across imputed data sets
#add factor scores to each imputed data set
#add factor scores to all data sets
for(i in 1:length(imps$imputations)){
  imps$imputations[[i]]=cbind(imps$imputations[[i]], fa.anals[[i]]$scores)
}

loads=lapply(fa.anals, loadings)
loads=lapply(loads, function(x) {class(x)="matrix"; return(x)})
#sort loadings before combining using Rubin's rules
loads=lapply(loads, function(x) x[,order(colnames(x))])
outloads=mi.pointest(loads)
outloads #compare to Table 3


#correlation of factor scores with underlying factors
rowMeans(sapply(fa.anals, extract.cors))


##PHASE I: How much does morality matter for politics?

#estimate effects of MFT variables on political outcomes
mods=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.conserv=mi.z(mods)
r2.conserv=mean(sapply(mods, function(x) summary(x)$r.sq))

# how does this compare to the predictive power of just demographic variables?
mods.dem = with(imps, lm(conserv ~ female + black + hispanic + otherrace + hs + somecoll + ba + maplus + ppincimp + age + age.sq + married + south + consrel + other.christian + norelig, weights=weight))
r2.conserv.dem=mean(sapply(mods.dem, function(x) summary(x)$r.sq))

## but is this just due to those on the extreme ends of the political spectrum?
mod.conserv.trim1 = with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight, subset=libcons!=1 & libcons !=7))
mod.conserv.trim2 = with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight, subset=party.rev !=1 & party.rev !=7))

#compare full sample and two trimmed samples side-by-side
r2.conserv.trim1=mean(sapply(mod.conserv.trim1, function(x) summary(x)$r.sq))
r2.conserv.trim2=mean(sapply(mod.conserv.trim2, function(x) summary(x)$r.sq))
bars=c(r2.conserv, r2.conserv.trim1, r2.conserv.trim2)
#how many are excluded in each trimmed sample? - note don't need to use imputations to get this number, since I excluded only those exactly 1 or 7, which does not vary across samples
sum(d$libcons==1 | d$libcons==7, na.rm=T)
sum(d$party.rev==1 | d$party.rev==7, na.rm=T)

#Figure 1
par(mar=c(4,5,3,3))
bp=barplot(bars, ylab=expression(paste("R"^2)), col=rep(c("lightblue", "goldenrod", "darkred"),2), las=1, ylim=c(0,max(bars)+max(bars)/10), las=2)
legend("topright", legend=c("Full sample", "No ideological extremes", "No strong party affiliations"), fill=c("lightblue", "goldenrod", "darkred"), cex=.8)



##PHASE II: Which moral differences predict political differences?
mods.conserv.latent=with(imps, lm(conserv ~ self.focus + order + other.focus, weights=weight))
mod.conserv.latent=mi.z(mods.conserv.latent)

#show that full model controls for these factors
cor(cbind(resid(mods[[1]]), imps$imputations[[1]]$self.focus, imps$imputations[[1]]$order, imps$imputations[[1]]$other.focus))

#table of regression results - Table 4
coefs=c(coef(mod.conserv.latent), coef(mod.conserv))
ses=c(sqrt(diag(vcov(mod.conserv.latent))), sqrt(diag(vcov(mod.conserv))))
coefmat=cbind(coefs, ses)
coefmat=cbind(coefmat, coefmat)
coefmat[5:dim(coefmat)[1],1:2]=NA
coefmat[1:4, 3:4]=NA
#trim out 2nd intercept row
coefmat=coefmat[!duplicated(rownames(coefmat)),]
coefmat

#figure to examine standardized coefficients
#for conservatism
bars=c(abs(coef(mod.conserv)[-1]), abs(coef(mod.conserv.latent)[-1]))
#set any non-sig. terms to 0
zvals.conserv = abs(coef(mod.conserv)/sqrt(diag(vcov(mod.conserv))))
zvals.latent = abs(coef(mod.conserv.latent)/sqrt(diag(vcov(mod.conserv.latent))))

#include marginally significant effects
ind=c(zvals.conserv[-1] <1.65, zvals.latent[-1] <1.65)
bars[ind]=0
names.subscales.sort=c("Divine authority", "Moral absolutism", "Nurturant parent", "Strict father", "Care", "Fairness", "Ingroup", "Authority", "Purity", "Conformity", "Tradition", "Benevolence", "Universalism", "Self-direction", "Stimulation", "Hedonism", "Achievement", "Power", "Security")
names.full.sort=c("Hunter", "Lakoff", "MFT", "Values")
colors=c(rep("darkred", 2), rep("goldenrod", 2), rep("blue", 5), rep("tan", 10))

par(mar=c(10,5,2,2))
bp=barplot(bars, names.arg=c(names.subscales.sort, "Self-focus", "Order", "Other-focus"), ylab="Standardized coefficient (absolute value)", col=c(colors, rep("forestgreen", 4)), las=3, ylim=c(0,max(bars)+max(bars)/9))
legend(0, 0.33, legend=c(names.full.sort, "common"), fill=c("darkred", "goldenrod", "blue", "tan", "forestgreen"))
abline(h=quantile(bars)["75%"], lty=2)
text(11, quantile(bars)["75%"], label="4th quartile", pos=3)

#how does this change when adding controls for demographics?
mods2=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security + female + black + hispanic + otherrace + hs + somecoll + ba + maplus + ppincimp + age + age.sq + married + south + consrel + other.christian + norelig, weights=weight))
mod.conserv2=mi.z(mods2)

mods.conserv.latent2=with(imps, lm(conserv ~ self.focus + order + other.focus + female + black + hispanic + otherrace + hs + somecoll + ba + maplus + ppincimp + age + age.sq + married + south + consrel + other.christian + norelig, weights=weight))
mod.conserv.latent2=mi.z(mods.conserv.latent2)

#plot unadjusted and adjusted coefs side-by-side - Figure 2
#unadjusted
bars=c(abs(coef(mod.conserv)[-1]), abs(coef(mod.conserv.latent)[-1]))
#set any non-sig. terms to 0
zvals.conserv = abs(coef(mod.conserv)/sqrt(diag(vcov(mod.conserv))))
zvals.latent = abs(coef(mod.conserv.latent)/sqrt(diag(vcov(mod.conserv.latent))))
ind=c(zvals.conserv[-1] <1.65, zvals.latent[-1] <1.65)
bars[ind]=0

bars2=c(abs(coef(mod.conserv2)[2:20]), abs(coef(mod.conserv.latent2)[2:4]))
#set any non-sig. terms to 0
zvals.conserv2 = abs(coef(mod.conserv2)[2:20]/sqrt(diag(vcov(mod.conserv2)[2:20,2:20])))
zvals.latent2 = abs(coef(mod.conserv.latent2)[2:4]/sqrt(diag(vcov(mod.conserv.latent2)[2:4,2:4])))
ind=c(zvals.conserv2 <1.96, zvals.latent2 <1.96)
bars2[ind]=0
bars=rbind(bars, bars2)

par(mar=c(10,5,2,2))
bp=barplot(bars, names.arg=c(names.subscales.sort, "Self-focus", "Order", "Other-focus"), ylab="Standardized coefficient (absolute value)", col=rep(c("lightblue", "darkblue"), length(bars[1,])), las=3, ylim=c(0,max(bars)+max(bars)/9), beside=T)
legend(0, 0.38, legend=c("Morality", "Adjusted morality"), fill=c("lightblue", "darkblue"), cex=.9)


## interpretive analyses - what do the results mean? ##
# estimate using individual scale items one at a time (to avoid controlling away what is common among them, which is precisely what the scale is capturing and hence what we are trying to explain)

#divine authority
imps$imputations=lapply(imps$imputations, function(x) {x$MR2.rev = 8-d$MR2; return(x)})
imps$imputations=lapply(imps$imputations, function(x) {x$MR3.rev = 8-d$MR3; return(x)})
mods.da=with(imps, lm(conserv ~ MR2.rev + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.da1=mi.z(mods.da)
mods.da=with(imps, lm(conserv ~ MR3.rev + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.da2=mi.z(mods.da)

coeftest(mod.da1)
coeftest(mod.da2)
#both items predict

#strict father
imps$imputations=lapply(imps$imputations, function(x) {x$L13 = d$L13; return(x)})
imps$imputations=lapply(imps$imputations, function(x) {x$L14 = d$L14; return(x)})
imps$imputations=lapply(imps$imputations, function(x) {x$L15 = d$L15; return(x)})
imps$imputations=lapply(imps$imputations, function(x) {x$L16 = d$L16; return(x)})
mods.sf=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + L13 + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.sf1=mi.z(mods.sf)
mods.sf=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + L14 + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.sf2=mi.z(mods.sf)
mods.sf=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + L15 + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.sf3=mi.z(mods.sf)
mods.sf=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + L16 + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.sf4=mi.z(mods.sf)

coeftest(mod.sf1)
coeftest(mod.sf2)
coeftest(mod.sf3)
coeftest(mod.sf4)
#here the predictors are discipline to build character, and sort of "tough love" needed to raise child right

#care
mods.care=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + q27 + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.care1=mi.z(mods.care)  #compassion is most crucial virtue
mods.care=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + q28 + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.care2=mi.z(mods.care) #never right to kill
mods.care=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + q52b + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.care3=mi.z(mods.care)
mods.care=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + q52c + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.care4=mi.z(mods.care)

coeftest(mod.care1)
coeftest(mod.care2)
coeftest(mod.care3)
coeftest(mod.care4)

#universalism
imps2=imps
imps2$imputations=lapply(imps2$imputations, function(x) cbind(x, revSV3=d$revSV3, revSV8=d$revSV8, revSV19=d$revSV19))
mods.univ=with(imps2, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + revSV3 + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.univ1=mi.z(mods.univ)
mods.univ=with(imps2, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + revSV8 + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.univ2=mi.z(mods.univ)
mods.univ=with(imps2, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + mfq.pure + sv.conform + sv.trad + sv.benev + revSV19 + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.univ3=mi.z(mods.univ)

coeftest(mod.univ1)
coeftest(mod.univ2)
coeftest(mod.univ3)
# so NOT the wanting to listen to those who are different and understand them
# is the equal treatment of others, and caring for nature

#purity
mods.pure=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + q35 + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.pure1=mi.z(mods.pure)
mods.pure=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + q36 + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.pure2=mi.z(mods.pure)
mods.pure=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + q52j + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.pure3=mi.z(mods.pure)
mods.pure=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + q52k + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.pure4=mi.z(mods.pure)

coeftest(mod.pure1) 
coeftest(mod.pure2)
coeftest(mod.pure3)
coeftest(mod.pure4)

#the only predictor is having some acts be wrong because they are unnatural, not the two disgust items
#try grouping disgust items together to make them stronger
imps$imputations=lapply(imps$imputations, function(x) {x$disgustitems = rowMeans(cbind(x$q35, x$q52k)); return(x)})
mods.pure=with(imps, lm(conserv ~ moralrelig + absolutism + lak_npfam + lak_sffam + mfq.harm + mfq.fair + mfq.ingroup + mfq.auth + disgustitems + sv.conform + sv.trad + sv.benev + sv.univ + sv.selfd + sv.stim + sv.hedon + sv.achieve + sv.power + sv.security, weights=weight))
mod.pure=mi.z(mods.pure)
coeftest(mod.pure) #still nothing with disgust items

#coefs and SEs for output - Table 5
getSE = function(x){sqrt(diag(vcov(x)))}
cfs=c(coef(mod.da1)["MR2.rev"], coef(mod.da2)["MR3.rev"], coef(mod.sf1)["L13"], coef(mod.sf2)["L14"], coef(mod.sf3)["L15"], coef(mod.sf4)["L16"], coef(mod.care1)["q27"], coef(mod.care2)["q28"], coef(mod.care3)["q52b"], coef(mod.care4)["q52c"], coef(mod.univ1)["revSV3"], coef(mod.univ2)["revSV8"], coef(mod.univ3)["revSV19"], coef(mod.pure1)["q35"], coef(mod.pure2)["q36"], coef(mod.pure3)["q52j"], coef(mod.pure4)["q52k"])
ses=c(getSE(mod.da1)["MR2.rev"], getSE(mod.da2)["MR3.rev"], getSE(mod.sf1)["L13"], getSE(mod.sf2)["L14"], getSE(mod.sf3)["L15"], getSE(mod.sf4)["L16"], getSE(mod.care1)["q27"], getSE(mod.care2)["q28"], getSE(mod.care3)["q52b"], getSE(mod.care4)["q52c"], getSE(mod.univ1)["revSV3"], getSE(mod.univ2)["revSV8"], getSE(mod.univ3)["revSV19"], getSE(mod.pure1)["q35"], getSE(mod.pure2)["q36"], getSE(mod.pure3)["q52j"], getSE(mod.pure4)["q52k"])
pvals=stars=2*pnorm(abs(cfs/ses), lower.tail=F)
stars=ifelse(pvals < .001, "***", ifelse(pvals < .01, "**", ifelse(pvals < .05, "*", " ")))

cbind(round(cfs, 2), round(ses, 2),stars)
